@@ -91,6 +91,12 @@ print.bridge <- function(x,...){
   print(x)
 }
 
+#' @export
+print.goldbricker <- function(x,...){
+  cat("Suggested reductions: Less than", x$threshold*100, "% of correlations are significantly different for the following pairs: \n")
+  print(x$suggested_reductions)
+}
+
 #' Plot "all.impact" objects
 #'
 #' Convenience function for generating impact plots
@@ -549,7 +555,7 @@ plot.bridge <- function(x, order=c("given","alphabetical", "value"), zscore=FALS
         ggplot2::geom_path() + ggplot2::geom_point() + ggplot2::xlab("") + ggplot2::ylab("") +
         ggplot2::facet_grid('~measure', scales="free")
     }
-    if(length(include)==1){g <- glist[[1]]
+    if(length(include)==1){g <- gridExtra::grid.arrange(glist[[1]])
     } else if(length(include)==2){gridExtra::grid.arrange(glist[[1]],glist[[2]], ncol=2)
     } else if(length(include)==3){gridExtra::grid.arrange(glist[[1]],glist[[2]],glist[[3]], ncol=3)
     } else if(length(include)==4){gridExtra::grid.arrange(glist[[1]],glist[[2]],glist[[3]],glist[[4]], ncol=4)
@@ -561,4 +567,17 @@ plot.bridge <- function(x, order=c("given","alphabetical", "value"), zscore=FALS
     }
 }
 
+#' @export
+plot.goldbricker <- function(x,...){
+  X <- Y <- Proportion_Unique <- NULL
+  mat <- x$proportion_matrix
+  xd <- yd <-  colnames(mat)
+  data <- expand.grid(X=xd, Y=yd)
+  data$Proportion_Unique <- as.vector(mat)
+  ggplot2::ggplot(data, ggplot2::aes(X, Y, z= Proportion_Unique)) + ggplot2::geom_tile(ggplot2::aes(fill = Proportion_Unique)) +
+    ggplot2::theme_bw() +
+    ggplot2::scale_fill_gradient(low="red", high="white") +
+    ggplot2::ggtitle("Proportion of Unique Correlations")
+}
+
 
